use bevy::math::uvec2;
use fastrand::Rng;

use crate::world::generators::utils::outline_floors;
use crate::world::generators::{MapGenerator, TMP_FLOOR_GROUP};
use crate::world::level_map::{Indexer, LevelMap, MapLayer, MapLayerType, MapTile};

pub struct DrunkardGenerator;
impl MapGenerator for DrunkardGenerator {
	fn generate(indexer: &Indexer, rng: &Rng) -> LevelMap {
		let map_width = indexer.width();
		let map_height = indexer.height();

		let mut iterations = rng.usize(700..1500);
		let mut floor = vec![0; map_width * map_height];

		let start = uvec2(map_width as u32 / 2, map_height as u32 / 2);
		let mut previous = start.clone();
		floor[indexer.index(start.x as usize, start.y as usize)] = TMP_FLOOR_GROUP;

		while iterations > 0 {
			iterations -= 1;
			let mut dx = 0;
			let mut dy = 0;

			match rng.u8(0..4) {
				0 => {
					dx += 1;
				}
				1 => {
					dx -= 1;
				}
				2 => {
					dy += 1;
				}
				3 => {
					dy -= 1;
				}
				_ => unreachable!(),
			}

			let target_x = (previous.x as i32 + dx).max(0) as usize;
			let target_y = (previous.y as i32 + dy).max(0) as usize;
			let target_idx = indexer.index(target_x, target_y);
			if indexer.index_within(target_idx) {
				floor[target_idx] = TMP_FLOOR_GROUP;
				previous = uvec2(target_x as u32, target_y as u32);
			}
		}

		let mut walls = vec![0; floor.len()];
		outline_floors(&floor, &mut walls, &indexer);

		LevelMap::new(
			map_width,
			map_height,
			start,
			vec![
				MapLayer::from_sized_list(
					map_width,
					map_height,
					floor
						.into_iter()
						.map(|tile| {
							if tile == 0 {
								None
							} else {
								Some(MapTile::new_floor(tile))
							}
						})
						.collect(),
					MapLayerType::Floor,
				),
				MapLayer::from_sized_list(
					map_width,
					map_height,
					walls
						.into_iter()
						.map(|tile| {
							if tile == 0 {
								None
							} else {
								Some(MapTile::new_floor(tile))
							}
						})
						.collect(),
					MapLayerType::Wall,
				),
			],
			Vec::new(),
		)
	}
}
